@@ -685,10 +685,13 @@ PluginSelector::PluginSelector( QHacc * engine, int pitype, QWidget * p, int d,
 	}
 	formater->setButton( nid );
 
+	connect( formater, SIGNAL( clicked( int ) ), SLOT( btnsel( int ) ) );
 }
 
 PluginSelector::~PluginSelector(){}
 
+void PluginSelector::btnsel( int ){ emit changedPI( getSelectedInfo() ); }
+
 void PluginSelector::setTitle( const QString& s ){ formater->setTitle( s ); }
 int PluginSelector::getButtonID() const { return formater->selectedId(); }
 PluginInfo PluginSelector::getSelectedInfo() const {
@@ -698,7 +701,7 @@ PluginInfo PluginSelector::getSelectedInfo() const {
 }
 
 /* * * * * * * * * * * * * * * * * * * */
-/* * * * * Impport/Export Base * * * * */
+/* * * * * Import/Export Base  * * * * */
 /* * * * * * * * * * * * * * * * * * * */
 HomeDlg::HomeDlg( QHacc * eng, int pitype, QWidget * p, const char * n )
 	: QDialog( p, n, true ){
@@ -906,6 +909,9 @@ QHaccImportDlg::QHaccImportDlg( QHacc * eng, QWidget * p,	const char * n )
 
 	arcFlag=new QCheckBox( i18n( "This is Archived Data" ), this );
 	pisel->setTitle( i18n( "Import Format" ) );	
+
+	connect( pisel, SIGNAL( changedPI( const PluginInfo& ) ),
+					 this, SLOT( newInfo( const PluginInfo& ) ) );
 }
 
 QHaccImportDlg::~QHaccImportDlg(){}
@@ -929,6 +935,17 @@ void QHaccImportDlg::dolayout(){
 	layout->addMultiCellWidget( cancel, row, row, 5, 9 );
 }
 
+void QHaccImportDlg::newInfo( const PluginInfo& inf ){
+	if( NONE==inf.target() ){
+		filename->setEnabled( false );
+		browsebtn->setEnabled( false );
+	}
+	else{
+		filename->setEnabled( true );
+		browsebtn->setEnabled( true );
+	}
+}
+
 QString QHaccImportDlg::home( bool * okay ) const {
 	QString ret=HomeDlg::home( okay );
 	if( arcFlag->isChecked() ) ret="ARC:"+ret;
