@@ -282,11 +282,17 @@ void QHaccTViewItem::paintCell( QPainter * p, const QColorGroup&, int c,
 			}
 			break;
 		case 4:
-			if( sumcompval>0 ) txt=mconv.convert( sumcompval, Engine, Preference );
+			if( sumcompval>0 ){
+				txt=mconv.convert( sumcompval, Engine, Preference );
+				if( myrow[QC::XTVOID].getb() ) txt=i18n( "VOID" );
+			}
 			else dowrite=false;
 			break;
 		case 5:
-			if( sumcompval<0 ) txt=mconv.convert( 0-sumcompval, Engine, Preference );
+			if( sumcompval<0 ){
+				txt=mconv.convert( 0-sumcompval, Engine, Preference );
+				if( myrow[QC::XTVOID].getb() ) txt=i18n( "VOID" );
+			}
 			else dowrite=false;
 			break;
 		default:
@@ -352,19 +358,19 @@ void QHaccTViewItem::updateA( const Account& newy ){
 }
 
 bool QHaccTViewItem::acceptDrop( QMimeSource * e ){
-	return( QHaccTDrag::canDecode( e ) );
+	//cout<<"tvi acceptdrop"<<endl;
+	return( QHaccTDrag::canDecode( e ) || QHaccADrag::canDecode( e ) );
 }
 
-
 void QHaccTViewItem::dropper( QDropEvent * e ){
-	return;
-
+	//cout<<"tvi dropper"<<endl;
 	if( myrow.isNull() ) return;
 
 	uint a;
 	Transaction t;
 	QHaccTable s;
 	if( QHaccTDrag::decode( e, t, s, a ) ){
+		//cout<<"decoded as a transaction"<<endl;
 		e->accept();
 		if( myrow[QC::XSACCTID]!=a ){
 			// moving a transaction from another account to this one
@@ -377,7 +383,9 @@ void QHaccTViewItem::dropper( QDropEvent * e ){
 	else if( QHaccADrag::decode( e, t ) ){
 		// if this is a double-entry transaction with only one
 		// split, change the other split to point to the dropped account
+		//cout<<"dropped an account on "<<myrow.toString()<<endl;
 		if( !( pairacct==0 || t[QC::AID]==myrow[QC::XSACCTID] ) ){
+			//cout<<"trying to resplit"<<endl;
 			QHaccTable spl( engine->getTSplits( myrow[QC::XTID].getu() ) );
 			TableSelect ts( QC::SACCTID, myrow[QC::XSACCTID], TableSelect::NE );
 			//for( uint i=0; i<spl.rows(); i++ ) cout<<spl[i].toString()<<endl;
@@ -394,7 +402,9 @@ void QHaccTViewItem::dropper( QDropEvent * e ){
 
 
 
-
+/*************************************************/
+/*************** QHaccView Class *****************/
+/*************************************************/
 
 
 QHaccView::QHaccView( QHacc * eng, QWidget * p, const char * n )
@@ -424,7 +434,7 @@ QHaccView::QHaccView( QHacc * eng, QWidget * p, const char * n )
 	//viewport()->setMouseTracking( true );
 	editor=0;
 	editorLPT=0;
-	//drec=QC::YREC;
+	isrecwin=false;
 
   QPopupMenu * popup=new QPopupMenu( this );
 	mempop=new QPopupMenu( this );
@@ -450,8 +460,9 @@ QHaccView::QHaccView( QHacc * eng, QWidget * p, const char * n )
 	popup->insertItem( i18n( "open mem editor" ), this, SLOT( edMTrans() ) );
 
 	setPopup( popup );
-	viewport()->setAcceptDrops( true );
+
 	setAcceptDrops( true );
+	viewport()->setAcceptDrops( true );
 
 	memtrans.reset( new QHaccTable( QC::NCOLS, QC::NCOLTYPES ) );
 	
@@ -543,10 +554,12 @@ void QHaccView::grabData(){
 		}
 	}
 
-	// remove transactions not in the current journal
-	tbl.deleteWhere( TableSelect( QC::XTLID, journal[QC::LID],
-																TableSelect::NE ) );
-
+	// remove transactions not in the current journal for non-recwins
+	if( ( isrecwin && !engine->getBP( "RECWINSHOWSALLJOURNALS" ) ) 
+			|| !isrecwin ){
+		tbl.deleteWhere( TableSelect( QC::XTLID, journal[QC::LID],
+																	TableSelect::NE ) );
+	}
 	// do any miscellaneous selections
 	auto_ptr<QHaccResultSet> t2=tbl.getWhere( selectors, rr );
 	temp=t2;
@@ -659,6 +672,7 @@ void QHaccView::setJournal( const Journal& l ){
 }
 
 void QHaccView::startDrag(){
+	//cout<<"am i here?"<<endl;
 	const Transaction& xt=getMoused();
 	if( !xt.isNull() ){
 		Transaction t=engine->splitXTrans( xt );
@@ -670,10 +684,12 @@ void QHaccView::startDrag(){
 }
 
 void QHaccView::contentsDragEnterEvent( QDragEnterEvent * e ){
-	if( QHaccTDrag::canDecode( e ) ) e->accept();
+	//cout<<"contentsdragenter"<<endl;
+	if( QHaccTDrag::canDecode( e ) || QHaccADrag::canDecode( e ) ) e->accept();
 }
 
 void QHaccView::contentsDropEvent( QDropEvent * e ){
+	//cout<<"contentsdropevent"<<endl;
 	QHaccTViewItem * tvi=( QHaccTViewItem * )itemAt( contentsToViewport( e->pos() ) );
 	if( tvi && !getRow( tvi ).isNull() ) tvi->dropper( e );
 	else {
@@ -963,12 +979,18 @@ void QHaccView::updT( const Transaction& xt ){
 		}
 		else addT( xt ); // we should have it, but don't...need to add it
 	}
-	else if( doHaveIt ) delete lvi;
+	else if( doHaveIt ){
+		unregisterItem( xt[QC::XSID].getu() );
+		delete lvi;
+	}
 }
 
 void QHaccView::remT( const Transaction& newy ){
 	QHaccListViewItem * avi=findRowItem( newy[QC::XSID].getu() );
-	if( avi )	delete avi;
+	if( avi ){
+		unregisterItem( newy[QC::XSID].getu() );
+		delete avi;
+	}
 }
 
 void QHaccView::edMTrans(){
@@ -981,11 +1003,13 @@ void QHaccView::addMTrans( int nid ){
 	const uint TID=nt.getu( QC::NTID );
 	Transaction tr=engine->getT( TID );
 	QHaccTable sp=engine->getTSplits( TID );
+	sp.updateWhere( TableSelect(), TableUpdate( QC::SRECO,
+																							TableCol( QC::NREC ) ) );
 
 	if( engine->isResolvable( tr, sp ) ){
 		// if this transaction is resolvable, just add it to the engine
-		sp.updateWhere( TableSelect(), TableUpdate( QC::SRECO,
-																								TableCol( QC::NREC ) ) );
+		sp.updateWhere( TableSelect( QC::SACCTID, 0 ), 
+										TableUpdate( QC::SACCTID, account[QC::AID] ) );
 
 		tr.set( QC::TDATE, TableCol( getIUMDate() ) );
 		tr.set( QC::TLID, journal[QC::LID] );
@@ -1092,6 +1116,7 @@ QHaccRecView::QHaccRecView( QHacc * e, QWidget * p,	const char * n )
 	connect( this, SIGNAL( clicked( QListViewItem * ) ), SLOT( recTrans() ) );
 	connect( this, SIGNAL( spacePressed( QListViewItem * ) ),
 					 this, SLOT( recTrans() ) );
+	isrecwin=true;
 }
 
 QHaccRecView::~QHaccRecView(){}
@@ -1131,3 +1156,4 @@ void QHaccRecView::ifinishEditor( TransactionEditor * ed ){
 void QHaccRecView::ifinishItem( QHaccTViewItem * i ){
 	i->setDrawDoubleEntry( false );
 }
+
