@@ -477,6 +477,8 @@ void QHaccWidget::updateAccount( const Account&, const Account& newy ){
 
 	if( !account.isNull() && account[QC::AID]!=newy[QC::AID] ) return;
 	if( journal.isNull() ) return;
+	account=newy;
+	
 	setCaption( engine->getSP( "TITLEBARNAME" )+" - "+journal[QC::LNAME].gets()+
 							"/"+engine->getFNameOfA( account ) );
 
@@ -585,7 +587,10 @@ void QHaccWidget::changePref( const QString&, QFont f ){
 }
 
 void QHaccWidget::changePref( const QString& s, QString v ){
-	if( s=="TITLEBARNAME" ) setCaption( v );
+	if( s=="TITLEBARNAME" ){
+		setCaption( v+" - "+journal[QC::LNAME].gets()+
+								"/"+engine->getFNameOfA( account ) );
+	}
 	else if( s=="CSYMBOL" || s=="ALTCSYMBOL" || s=="CURRENCYSEPARATOR" ){
 		Account a=chooser->getSelected();
 		updateAccount( a, a );
