@@ -21,6 +21,7 @@
 #include "qhacc.h"
 #include "qhacctable.h"
 #include "qhaccutils.h"
+#include "metadialog.h"
 #include "splitdialog.h"
 #include "guiconstants.h"
 #include "qhacclineedits.h"
@@ -36,8 +37,8 @@
 #include <set>
 using namespace std;
 
-//#include <mpatrol.h>
 #include <klocale.h>
+//#include <mpatrol.h>
 
 TransactionEditor::TransactionEditor( QHacc * eng, const Account& acct,
 																			const Journal& l, bool de, QWidget * p,
@@ -65,15 +66,15 @@ TransactionEditor::TransactionEditor( QHacc * eng, const Account& acct,
 	memo=0;
 	date=0;
 	reco=0;
-	split=okay=0;
+	split=okay=meta=0;
 	globalTA=engine->getBP( "GLOBALTYPEAHEAD" );
 	nocaselkp=engine->getBP( "AUTOCOMPNOCASE" );
 	autocomp=engine->getIP( "AUTOCOMPLETION" );
-	//failedPayee="";
 
   // transaction types
   num=new QHaccChoiceEdit( true, this );
 	num->setEmptyText( i18n( "<num>" ) );
+	num->ignoreCase( nocaselkp );
 
   // date field  
   date=new QHaccDateEdit( this );
@@ -114,12 +115,14 @@ TransactionEditor::TransactionEditor( QHacc * eng, const Account& acct,
 		
     split=new QPushButton( i18n( "Split" ), this );
     connect( split, SIGNAL( clicked() ), SLOT( openSplitEditor() ) );
+
+    meta=new QPushButton( i18n( "More" ), this );
+    connect( meta, SIGNAL( clicked() ), SLOT( openMetaEditor() ) );
 		
 		popDEA();
 		connect( engine, SIGNAL( addedA( const Account& ) ), SLOT( popDEA() ) );
 		connect( engine, SIGNAL( removedA( const Account& ) ), SLOT( popDEA() ) );
-		connect( engine, SIGNAL( updatedA( const Account&, const Account& ) ), 
-						 SLOT( popDEA() ) );
+		// updatedA is handled by updateAccount, below
   }
 	else setTabOrder( payee, memo );
   
@@ -134,6 +137,8 @@ TransactionEditor::TransactionEditor( QHacc * eng, const Account& acct,
 					 SLOT( changeP( const QString&, bool ) ) );
 	connect( engine, SIGNAL( changedP( const QString&, int ) ),
 					 SLOT( changeP( const QString&, int ) ) );
+	connect( engine, SIGNAL( updatedA( const Account &, const Account& ) ),
+					 this, SLOT( updateAccount( const Account &, const Account& ) ) );
 }
 
 TransactionEditor::~TransactionEditor(){
@@ -145,23 +150,19 @@ TransactionEditor::~TransactionEditor(){
 void TransactionEditor::popDEA(){
 	if( doubleEntry ){
 		deAccount->clear();
-
+		
 		// we need to fill in the account names for the double entry lookup
 		vector<int> tgs;
-		tgs.push_back( QC::AID );
-		tgs.push_back( QC::APID );
-		tgs.push_back( QC::ANAME );
+		for( int i=0; i<AccountArea::COLS; i++ )
+			tgs.push_back( AccountArea::GRABCOLS[i] );
+		
+		const bool useanums=engine->getBP( "USEANUMSFORNAMES" );
 		
 		auto_ptr<QHaccResultSet> accts=engine->getAs( TableGet( tgs ) );
 		uint rr=accts->rows();
 		for( uint i=0; i<rr; i++ ){
-			Account a=accts->at( i );
-			if( a[1]==0 ) deAccount->insertItem( a[2].gets() );
-			else{
-				deAccount->insertItem( engine->getFNameOfA( a[1].getu() )+QC::ASEP+
-															 a[2].gets() );
-			}
-		}		
+			deAccount->insertItem( AccountAreaFrame::displayAName( engine, accts->at( i ), useanums ) );
+		}
 	}
 }
 
@@ -196,16 +197,25 @@ void TransactionEditor::reLayout( QHeader * header, const QRect& r ){
 
 		// go to line 2
 		y+=( h+1 );
-		idx=0;
-		QWidget * twdgs[]={ split, deAccount, memo, okay };
-		for( int i=0; i<6; i++ ){
-			int x=header->sectionPos( i )+1;
-			int w=header->sectionSize( i )-1;
-			if( idx==0 || idx==3 ) w+=header->sectionSize( ++i );
-			if( idx==3 ) w+=header->sectionSize( ++i )-1;
-			twdgs[idx]->setGeometry( x, y, w, h );
-			idx++;
-		}
+		int x=header->sectionPos( 0 )+1;
+		int w=header->sectionSize( 0 )+header->sectionSize( 1 )-2;
+		split->setGeometry( x, y, w, h );
+
+		x=header->sectionPos( 2 )+1;
+		w=header->sectionSize( 2 )-1;
+		deAccount->setGeometry( x, y, w, h );
+
+		x=header->sectionPos( 3 )+1;
+		w=header->sectionSize( 3 )-1;
+		memo->setGeometry( x, y, w, h );
+
+		x=header->sectionPos( 4 )+1;
+		w=header->sectionSize( 4 )+header->sectionSize( 5 )-2;
+		okay->setGeometry( x, y, w, h );
+
+		x=header->sectionPos( 6 )+1;
+		w=header->sectionSize( 6 )-1;
+		meta->setGeometry( x, y, w, h );
 	}
 	else{
 		QWidget * wdgs[]={ num, date, payee, memo, credit, debit, reco };
@@ -265,20 +275,27 @@ void TransactionEditor::prepare( const Transaction& t, bool forEdit ){
 	num->insertStringList( sts );
 	num->insertItem( t[QC::TNUM].gets() );
 
-
 	Split s;
 	if( edit ){
 		QHaccTable splits=engine->getTSplits( t.getu( QC::TID ) );
 		s=splits.getWhere( TableSelect( QC::SACCTID, account[QC::AID] ) );
 		// named trans might not have a valid split for this account
-		if( s.isNull() ) s=TableRow( QC::SCOLS );
+		if( s.isNull() ){
+			s=TableRow( QC::SCOLS );
+			metatax=account.getb( QC::ATAXED );
+		}
+		else metatax=s[QC::STAXABLE].getb();
 
+		metavoid=t[QC::TVOID].getb();
+		
 		named=( t[QC::TTYPE]==QC::MEMORIZED );
 		if( !named ) reco->setChecked( s[QC::SRECO]!=QC::NREC );
 	}
 	else{
 		s=TableRow( QC::SCOLS );
 		named=false;
+		metatax=account.getb( QC::ATAXED );
+		metavoid=false;
 	}
 
 	origT=model=engine->makeXTrans( t, s );
@@ -292,10 +309,6 @@ void TransactionEditor::prepare( const Transaction& t, bool forEdit ){
 									this, SLOT( moveInCredit( const QString& ) ) );
 			disconnect( debit, SIGNAL( textChanged( const QString& ) ),
 									this, SLOT( moveInDebit( const QString& ) ) );
-			//disconnect( credit, SIGNAL( arrowPressed( bool ) ),
-			//					this, SLOT( moveInSum( bool ) ) );
-			//disconnect( debit, SIGNAL( arrowPressed( bool ) ),
-			//					this, SLOT( moveInSum( bool ) ) );
 			connected=false;
 		}
 	}
@@ -306,8 +319,7 @@ void TransactionEditor::prepare( const Transaction& t, bool forEdit ){
 			connect( credit, SIGNAL( textChanged( const QString& ) ),
 							 this, SLOT( moveInCredit( const QString& ) ) );
 			connect( debit, SIGNAL( textChanged( const QString& ) ),
-							 this, SLOT( moveInDebit( const QString& ) ) );
-			
+							 this, SLOT( moveInDebit( const QString& ) ) );			
 			connected=true;
 		}
 	}
@@ -329,6 +341,12 @@ void TransactionEditor::setModel( const Transaction& t, bool convit ){
 	credit->blockSignals( true );
 	debit->blockSignals( true );
 
+	Split marks;
+	Transaction junk;
+	engine->splitXTrans( t, junk, marks );
+	QString shares, price;
+	bool ismkt=Utils::isMarket( marks, &shares, &price );
+
 	const MonCon& conv=engine->converter();
 	if( convit ) summer=conv.convert( summer );
 	else model.set( QC::XSSUM, conv.convert( summer, Engine, Preference ) );
@@ -336,10 +354,12 @@ void TransactionEditor::setModel( const Transaction& t, bool convit ){
 	//cout<<"settled on "<<summer<<", but model is "<<model.gets( QC::XSSUM )<<endl;
 	if( summer.startsWith( "-" ) ){
 		credit->setText( "" );
-		debit->setText( summer.mid( 1 ) );
+		if( ismkt )	debit->setText( shares.mid( 1 )+"@"+price );
+		else debit->setText( summer.mid( 1 ) );
 	}
 	else{
 		if( summer==QC::REMAINDERVAL ) credit->setText( "" );
+		else if( ismkt ) credit->setText( shares+"@"+price );
 		else credit->setText( summer );
 		debit->setText( "" );
 	}
@@ -401,15 +421,15 @@ void TransactionEditor::fillInForPayee( const QString& smodel ){
 		uint rr=0;
 		auto_ptr<QHaccResultSet> rslt=engine->getWhere( XTRANS, crit, rr );
 		QHaccTableIndex sacctid( rslt.get(), QC::XSACCTID, CTUINT );
-		cout<<rr<<" transactions returned"<<endl;
-		for( uint i=0; i<rr; i++ )
-			cout<<"  "<<rslt->at( sacctid[i] ).toString()<<endl;
-		cout<<"--"<<endl;
+		//cout<<rr<<" transactions returned"<<endl;
+		//for( uint i=0; i<rr; i++ )
+		//cout<<"  "<<rslt->at( sacctid[i] ).toString()<<endl;
+		//cout<<"--"<<endl;
 
 
 		uint spos=sacctid.starts( account[QC::AID] );
 		uint epos=sacctid.ends( account[QC::AID] );
-		cout<<"s/e="<<spos<<"/"<<epos<<endl;
+		//cout<<"s/e="<<spos<<"/"<<epos<<endl;
 		
 		// keep track of tids to ignore when adding splits...
 		set<uint> badtids;
@@ -491,6 +511,8 @@ void TransactionEditor::conditionallyAccept(){
 	// (valid accounts and the final set of adjustments)
 	// and if we do, call accept
 	bool good=true;
+	bool ismkt=false;
+	QString mkttxt; // we'll need these for market transactions
 
 	// don't convert the values at this point, just the currency separator
 	//if(!credit->text().isEmpty()) cout<<"c="<<credit->text()<<endl;
@@ -500,7 +522,37 @@ void TransactionEditor::conditionallyAccept(){
 	int db=conv.converti( debit->text(), PrefSep, EngSep );
 	//cout<<"cr="<<cr<<"; db="<<db<<endl;
 
-	if( cr+db==0 ) return;
+	// if we don't get a good value, check to see if we have a market transaction
+	if( cr+db==0 ){
+		QString shares, price;
+
+		// FIXME: worry about correct formatting here!
+		mkttxt="market("+credit->text()+")";
+		if( Utils::isMarket( mkttxt, shares, price ) ){
+			ismkt=true;
+			// we have a market transactions, but it is still localized by the user
+			// we need to extract the values and multiply them together before we 
+			// can worry about converting the results
+			QString str=QString::number( shares.toFloat()*price.toFloat(),
+																	 'f', QC::DECIMALS );
+			cr+=( conv.converti( str, PrefSep, EngSep ) );
+		}
+		else{
+			mkttxt="market(-"+debit->text()+")";
+			if( Utils::isMarket( mkttxt, shares, price ) ){
+				ismkt=true;
+
+				// we need to flip the shares sign, because
+				// we added a - for the isMarket check
+				QString str=QString::number( -( shares.toFloat() )*price.toFloat(),
+																		 'f', QC::DECIMALS );
+				db+=( conv.converti( str, PrefSep, EngSep ) );
+			}
+		}
+	}
+
+	// if we still don't have a good value, something is wrong
+	if( cr+db==0 && !metavoid ) return;
 
 	if( doubleEntry ){
 		// check that the deAccount field is filled in
@@ -516,6 +568,7 @@ void TransactionEditor::conditionallyAccept(){
 				// we'll do the MonCon conversion in accept()
 				s.set( QC::SSUM, TableCol( conv.convert( db-cr, Engine, Engine ) ) );
 				s.set( QC::SACCTID, acct[QC::AID] );
+				s.set( QC::STAXABLE, acct[QC::ATAXED] );
 
 				//cout<<"tsplits"<<endl;
 				//for( uint i=0; i<tsplits->rows(); i++ ) 
@@ -540,6 +593,8 @@ void TransactionEditor::conditionallyAccept(){
 		Split s( QC::SCOLS );
 		s.set( QC::SSUM, TableCol( conv.convert( cr-db, Engine, Engine ) ) );
 		s.set( QC::SACCTID, account[QC::AID] );
+		s.set( QC::STAXABLE, metatax );
+		if( ismkt )	s.set( QC::SMETA, mkttxt );
 			
 		// if the reco box is checked, set the reconcile flag accordingly
 		if( reco->isChecked() )	s.set( QC::SRECO, TableCol( recstate ) );
@@ -576,6 +631,7 @@ void TransactionEditor::accept(){
 	Transaction t=engine->splitXTrans( model );
 	t.set( QC::TLID, journal[QC::LID] );
 	t.set( QC::TTYPE, TableCol( QC::REGULAR ) );
+	t.set( QC::TVOID, metavoid );
 
 	const MonCon& conv=engine->converter();
 
@@ -586,11 +642,11 @@ void TransactionEditor::accept(){
 		const QString& sum=s[QC::SSUM].gets();
 
 		if( !( sum==QC::REMAINDERVAL || sum.find( "%" )>-1
-					 || sum=="p" || sum=="i" ) )
+					 || sum=="p" || sum=="i" ) ){
 			// we've already converted the currency separator, but we
 			// still need to convert the values (if they are values)
 			s.set( QC::SSUM, conv.convert( sum, PrefVal, EngVal ) );
-
+		}
 		t2+=s;
 	}
 
@@ -669,6 +725,15 @@ void TransactionEditor::openSplitEditor(){
 	else setModel( model );
 }
 
+void TransactionEditor::openMetaEditor(){
+	MetaDialog md( metatax, metavoid, this );
+	if( md.exec() ){
+		metavoid=md.isVoid();
+		metatax=md.isTaxable();
+	}
+}
+
+
 void TransactionEditor::keyPressEvent( QKeyEvent * qke ){
 	if( qke->key()==Key_Escape ){
 		qke->accept();
@@ -694,3 +759,19 @@ void TransactionEditor::changeP( const QString& pref, bool b ){
 	}
 	else if( pref=="GLOBALTYPEAHEAD" ) globalTA=b;
 }
+
+void TransactionEditor::updateAccount( const Account&, const Account& newy ){
+	if( newy[QC::AID]==account[QC::AID] ){
+		account=newy;
+		
+		const QString str( num->text() );
+
+		QStringList sts=QStringList::split( " ", account.gets( QC::ATRANSNUMS ),
+																				false );
+		num->clear();
+		num->insertStringList( sts );
+		num->insertItem( model[QC::TNUM].gets() );
+		num->setText( str );
+	}
+	else popDEA();
+}
